@@ -34,6 +34,8 @@
  */
 
 #include <stdint.h>
+#include "lflow-cache.h"
+#include "lflow-conj-ids.h"
 #include "openvswitch/hmap.h"
 #include "openvswitch/uuid.h"
 #include "openvswitch/list.h"
@@ -46,6 +48,7 @@ struct hmap_node;
 struct sbrec_chassis;
 struct sbrec_dhcp_options_table;
 struct sbrec_dhcpv6_options_table;
+struct sbrec_load_balancer;
 struct sbrec_logical_flow_table;
 struct sbrec_mac_binding_table;
 struct sbrec_datapath_binding;
@@ -60,9 +63,9 @@ struct uuid;
  * you make any changes. */
 #define OFTABLE_PHY_TO_LOG            0
 #define OFTABLE_LOG_INGRESS_PIPELINE  8 /* First of LOG_PIPELINE_LEN tables. */
-#define OFTABLE_REMOTE_OUTPUT        32
-#define OFTABLE_LOCAL_OUTPUT         33
-#define OFTABLE_CHECK_LOOPBACK       34
+#define OFTABLE_REMOTE_OUTPUT        37
+#define OFTABLE_LOCAL_OUTPUT         38
+#define OFTABLE_CHECK_LOOPBACK       39
 #define OFTABLE_LOG_EGRESS_PIPELINE  40 /* First of LOG_PIPELINE_LEN tables. */
 #define OFTABLE_SAVE_INPORT          64
 #define OFTABLE_LOG_TO_PHY           65
@@ -70,15 +73,15 @@ struct uuid;
 #define OFTABLE_MAC_LOOKUP           67
 #define OFTABLE_CHK_LB_HAIRPIN       68
 #define OFTABLE_CHK_LB_HAIRPIN_REPLY 69
-#define OFTABLE_CT_SNAT_FOR_VIP      70
-
-/* The number of tables for the ingress and egress pipelines. */
-#define LOG_PIPELINE_LEN 24
+#define OFTABLE_CT_SNAT_HAIRPIN      70
+#define OFTABLE_GET_FDB              71
+#define OFTABLE_LOOKUP_FDB           72
 
 enum ref_type {
     REF_TYPE_ADDRSET,
     REF_TYPE_PORTGROUP,
-    REF_TYPE_PORTBINDING
+    REF_TYPE_PORTBINDING,
+    REF_TYPE_MC_GROUP
 };
 
 struct ref_lflow_node {
@@ -102,6 +105,9 @@ struct lflow_ref_list_node {
     struct ovs_list list_node; /* node in lflow_ref_node.lflow_ref_head. */
     struct hmap_node hmap_node; /* node in ref_lflow_node.lflow_uuids. */
     struct uuid lflow_uuid;
+    size_t ref_count; /* Reference count of the resource by this lflow.
+                         Currently used for the resource type REF_TYPE_ADDRSET
+                         only, and for other types it is always 0. */
     struct ref_lflow_node *rlfn;
 };
 
@@ -129,6 +135,8 @@ struct lflow_ctx_in {
     struct ovsdb_idl_index *sbrec_logical_flow_by_logical_datapath;
     struct ovsdb_idl_index *sbrec_logical_flow_by_logical_dp_group;
     struct ovsdb_idl_index *sbrec_port_binding_by_name;
+    struct ovsdb_idl_index *sbrec_fdb_by_dp_key;
+    const struct sbrec_port_binding_table *port_binding_table;
     const struct sbrec_dhcp_options_table *dhcp_options_table;
     const struct sbrec_dhcpv6_options_table *dhcpv6_options_table;
     const struct sbrec_datapath_binding_table *dp_binding_table;
@@ -136,13 +144,15 @@ struct lflow_ctx_in {
     const struct sbrec_logical_flow_table *logical_flow_table;
     const struct sbrec_logical_dp_group_table *logical_dp_group_table;
     const struct sbrec_multicast_group_table *mc_group_table;
+    const struct sbrec_fdb_table *fdb_table;
     const struct sbrec_chassis *chassis;
     const struct sbrec_load_balancer_table *lb_table;
     const struct hmap *local_datapaths;
     const struct shash *addr_sets;
     const struct shash *port_groups;
     const struct sset *active_tunnels;
-    const struct sset *local_lport_ids;
+    const struct sset *related_lport_ids;
+    const struct hmap *chassis_tunnels;
 };
 
 struct lflow_ctx_out {
@@ -150,32 +160,56 @@ struct lflow_ctx_out {
     struct ovn_extend_table *group_table;
     struct ovn_extend_table *meter_table;
     struct lflow_resource_ref *lfrr;
-    struct hmap *lflow_cache_map;
-    uint32_t *conj_id_ofs;
-    bool conj_id_overflow;
+    struct lflow_cache *lflow_cache;
+    struct conj_ids *conj_ids;
+    struct hmap *lflows_processed;
+    struct simap *hairpin_lb_ids;
+    struct id_pool *hairpin_id_pool;
 };
 
+struct lflow_processed_node {
+    struct hmap_node hmap_node; /* In ed_type_lflow_output.lflows_processed. */
+    struct uuid lflow_uuid;
+};
+void lflows_processed_destroy(struct hmap *lflows_processed);
+
 void lflow_init(void);
-void  lflow_run(struct lflow_ctx_in *, struct lflow_ctx_out *);
+void lflow_run(struct lflow_ctx_in *, struct lflow_ctx_out *);
+void lflow_handle_cached_flows(struct lflow_cache *,
+                               const struct sbrec_logical_flow_table *);
 bool lflow_handle_changed_flows(struct lflow_ctx_in *, struct lflow_ctx_out *);
 bool lflow_handle_changed_ref(enum ref_type, const char *ref_name,
                               struct lflow_ctx_in *, struct lflow_ctx_out *,
                               bool *changed);
+
+struct addr_set_diff {
+    struct expr_constant_set *added;
+    struct expr_constant_set *deleted;
+};
+bool lflow_handle_addr_set_update(const char *as_name, struct addr_set_diff *,
+                                  struct lflow_ctx_in *,
+                                  struct lflow_ctx_out *,
+                                  bool *changed);
+
 void lflow_handle_changed_neighbors(
     struct ovsdb_idl_index *sbrec_port_binding_by_name,
     const struct sbrec_mac_binding_table *,
     const struct hmap *local_datapaths,
     struct ovn_desired_flow_table *);
 bool lflow_handle_changed_lbs(struct lflow_ctx_in *, struct lflow_ctx_out *);
+bool lflow_handle_changed_fdbs(struct lflow_ctx_in *, struct lflow_ctx_out *);
 void lflow_destroy(void);
 
-void lflow_cache_init(struct hmap *);
-void lflow_cache_destroy(struct hmap *);
-
 bool lflow_add_flows_for_datapath(const struct sbrec_datapath_binding *,
+                                  const struct sbrec_load_balancer **dp_lbs,
+                                  size_t n_dp_lbs,
                                   struct lflow_ctx_in *,
                                   struct lflow_ctx_out *);
 bool lflow_handle_flows_for_lport(const struct sbrec_port_binding *,
                                   struct lflow_ctx_in *,
                                   struct lflow_ctx_out *);
+bool lflow_handle_changed_mc_groups(struct lflow_ctx_in *,
+                                    struct lflow_ctx_out *);
+bool lflow_handle_changed_port_bindings(struct lflow_ctx_in *,
+                                        struct lflow_ctx_out *);
 #endif /* controller/lflow.h */
