@@ -21,6 +21,9 @@
 #include "lib/smap.h"
 #include "lib/simap.h"
 
+/* OVN includes. */
+#include "lib/ovn-util.h"
+
 struct sbrec_datapath_binding;
 struct sbrec_port_binding;
 struct sbrec_chassis;
@@ -41,6 +44,7 @@ struct local_datapath {
     struct hmap_node hmap_node;
     const struct sbrec_datapath_binding *datapath;
     bool is_switch;
+    bool is_transit_switch;
 
     /* The localnet port in this datapath, if any (at most one is allowed). */
     const struct sbrec_port_binding *localnet_port;
@@ -120,14 +124,6 @@ void tracked_datapath_lport_add(const struct sbrec_port_binding *,
                                 struct hmap *tracked_datapaths);
 void tracked_datapaths_destroy(struct hmap *tracked_datapaths);
 
-/* Must be a bit-field ordered from most-preferred (higher number) to
- * least-preferred (lower number). */
-enum chassis_tunnel_type {
-    GENEVE = 1 << 2,
-    STT    = 1 << 1,
-    VXLAN  = 1 << 0
-};
-
 /* Maps from a chassis to the OpenFlow port number of the tunnel that can be
  * used to reach that chassis. */
 struct chassis_tunnel {
@@ -154,5 +150,10 @@ bool get_chassis_tunnel_ofport(const struct hmap *chassis_tunnels,
                                ofp_port_t *ofport);
 
 void chassis_tunnels_destroy(struct hmap *chassis_tunnels);
+void local_datapath_memory_usage(struct simap *usage);
+void add_local_datapath_external_port(struct local_datapath *ld,
+                                      char *logical_port, const void *data);
+void remove_local_datapath_external_port(struct local_datapath *ld,
+                                         char *logical_port);
 
 #endif /* controller/local_data.h */
