@@ -52,15 +52,17 @@ void ofctrl_init(struct ovn_extend_table *group_table,
 void ofctrl_run(const struct ovsrec_bridge *br_int,
                 struct shash *pending_ct_zones);
 enum mf_field_id ofctrl_get_mf_field_id(void);
-void ofctrl_put(struct ovn_desired_flow_table *,
+void ofctrl_put(struct ovn_desired_flow_table *lflow_table,
+                struct ovn_desired_flow_table *pflow_table,
                 struct shash *pending_ct_zones,
                 const struct sbrec_meter_table *,
-                int64_t nb_cfg,
-                bool flow_changed);
+                uint64_t nb_cfg,
+                bool lflow_changed,
+                bool pflow_changed);
 bool ofctrl_can_put(void);
 void ofctrl_wait(void);
 void ofctrl_destroy(void);
-int64_t ofctrl_get_cur_cfg(void);
+uint64_t ofctrl_get_cur_cfg(void);
 
 void ofctrl_ct_flush_zone(uint16_t zone_id);
 
@@ -69,16 +71,34 @@ char *ofctrl_inject_pkt(const struct ovsrec_bridge *br_int,
                         const struct shash *port_groups);
 
 /* Flow table interfaces to the rest of ovn-controller. */
+
+/* Information of IP of an address set used to track a flow that is generated
+ * from a logical flow referencing address set(s). */
+struct addrset_info {
+    const char *name; /* The address set's name. */
+    struct in6_addr ip; /* An IP in the address set. */
+    struct in6_addr mask; /* The mask of the IP. */
+};
 void ofctrl_add_flow(struct ovn_desired_flow_table *, uint8_t table_id,
                      uint16_t priority, uint64_t cookie,
                      const struct match *, const struct ofpbuf *ofpacts,
                      const struct uuid *);
 
-void ofctrl_add_or_append_flow(struct ovn_desired_flow_table *desired_flows,
+void ofctrl_add_or_append_flow(struct ovn_desired_flow_table *,
                                uint8_t table_id, uint16_t priority,
-                               uint64_t cookie, const struct match *match,
+                               uint64_t cookie, const struct match *,
                                const struct ofpbuf *actions,
-                               const struct uuid *sb_uuid);
+                               const struct uuid *sb_uuid,
+                               uint32_t meter_id,
+                               const struct addrset_info *);
+
+void ofctrl_add_flow_metered(struct ovn_desired_flow_table *desired_flows,
+                             uint8_t table_id, uint16_t priority,
+                             uint64_t cookie, const struct match *match,
+                             const struct ofpbuf *actions,
+                             const struct uuid *sb_uuid,
+                             uint32_t meter_id,
+                             const struct addrset_info *);
 
 /* Removes a bundles of flows from the flow table for a specific sb_uuid. The
  * flows are removed only if they are not referenced by any other sb_uuid(s).
@@ -104,18 +124,26 @@ void ofctrl_flood_remove_flows(struct ovn_desired_flow_table *,
                                struct hmap *flood_remove_nodes);
 void ofctrl_flood_remove_add_node(struct hmap *flood_remove_nodes,
                                   const struct uuid *sb_uuid);
+bool ofctrl_remove_flows_for_as_ip(struct ovn_desired_flow_table *,
+                                   const struct uuid *lflow_uuid,
+                                   const struct addrset_info *,
+                                   size_t expected_count);
+
 void ovn_desired_flow_table_init(struct ovn_desired_flow_table *);
 void ovn_desired_flow_table_clear(struct ovn_desired_flow_table *);
 void ovn_desired_flow_table_destroy(struct ovn_desired_flow_table *);
 
-void ofctrl_check_and_add_flow(struct ovn_desired_flow_table *,
-                               uint8_t table_id, uint16_t priority,
-                               uint64_t cookie, const struct match *,
-                               const struct ofpbuf *ofpacts,
-                               const struct uuid *, bool log_duplicate_flow);
+void ofctrl_check_and_add_flow_metered(struct ovn_desired_flow_table *,
+                                       uint8_t table_id, uint16_t priority,
+                                       uint64_t cookie, const struct match *,
+                                       const struct ofpbuf *ofpacts,
+                                       const struct uuid *, uint32_t meter_id,
+                                       const struct addrset_info *,
+                                       bool log_duplicate_flow);
 
 
 bool ofctrl_is_connected(void);
 void ofctrl_set_probe_interval(int probe_interval);
+void ofctrl_get_memory_usage(struct simap *usage);
 
 #endif /* controller/ofctrl.h */
