@@ -37,6 +37,7 @@
 #include "openvswitch/ofp-parse.h"
 #include "ovn-controller.h"
 #include "lib/chassis-index.h"
+#include "lib/mcast-group-index.h"
 #include "lib/ovn-sb-idl.h"
 #include "lib/ovn-util.h"
 #include "physical.h"
@@ -63,6 +64,7 @@ static void
 load_logical_ingress_metadata(const struct sbrec_port_binding *binding,
                               const struct zone_ids *zone_ids,
                               struct ofpbuf *ofpacts_p);
+static int64_t get_vxlan_port_key(int64_t port_key);
 
 /* UUID to identify OF flows not associated with ovsdb rows. */
 static struct uuid *hc_uuid = NULL;
@@ -160,8 +162,9 @@ put_encapsulation(enum mf_field_id mff_ovn_geneve,
     } else if (tun->type == VXLAN) {
         uint64_t vni = datapath->tunnel_key;
         if (!is_ramp_switch) {
-            /* Only some bits are used for regular tunnels. */
-            vni |= (uint64_t) outport << 12;
+            /* Map southbound 16-bit port key to limited 12-bit range
+             * available for VXLAN, which differs for multicast groups. */
+            vni |= get_vxlan_port_key(outport) << 12;
         }
         put_load(vni, MFF_TUN_ID, 0, 24, ofpacts);
     } else {
@@ -832,7 +835,7 @@ put_local_common_flows(uint32_t dp_key,
         put_stack(MFF_IN_PORT, ofpact_put_STACK_POP(ofpacts_p));
         ofctrl_check_and_add_flow_metered(flow_table, OFTABLE_SAVE_INPORT, 100,
                                           0, &match, ofpacts_p, hc_uuid,
-                                          NX_CTLR_NO_METER, false);
+                                          NX_CTLR_NO_METER, NULL, false);
     }
 }
 
@@ -955,12 +958,12 @@ consider_port_binding(struct ovsdb_idl_index *sbrec_port_binding_by_name,
             || ha_chassis_group_is_active(binding->ha_chassis_group,
                                           active_tunnels, chassis))) {
 
-        /* Table 33, priority 100.
+        /* Table 38, priority 100.
          * =======================
          *
          * Implements output to local hypervisor.  Each flow matches a
          * logical output port on the local hypervisor, and resubmits to
-         * table 34.  For ports of type "chassisredirect", the logical
+         * table 39.  For ports of type "chassisredirect", the logical
          * output port is changed from the "chassisredirect" port to the
          * underlying distributed port. */
 
@@ -1007,7 +1010,7 @@ consider_port_binding(struct ovsdb_idl_index *sbrec_port_binding_by_name,
                 put_load(zone_ids.snat, MFF_LOG_SNAT_ZONE, 0, 32, ofpacts_p);
             }
 
-            /* Resubmit to table 34. */
+            /* Resubmit to table 39. */
             put_resubmit(OFTABLE_CHECK_LOOPBACK, ofpacts_p);
         }
 
@@ -1066,11 +1069,7 @@ consider_port_binding(struct ovsdb_idl_index *sbrec_port_binding_by_name,
     } else {
         ofport = local_binding_get_lport_ofport(local_bindings,
                                                 binding->logical_port);
-        const char *requested_chassis = smap_get(&binding->options,
-                                                 "requested-chassis");
-        if (ofport && requested_chassis && requested_chassis[0] &&
-            strcmp(requested_chassis, chassis->name) &&
-            strcmp(requested_chassis, chassis->hostname)) {
+        if (ofport && !lport_can_bind_on_this_chassis(chassis, binding)) {
             /* Even though there is an ofport for this port_binding, it is
              * requested on a different chassis. So ignore this ofport.
              */
@@ -1312,7 +1311,7 @@ consider_port_binding(struct ovsdb_idl_index *sbrec_port_binding_by_name,
 
     } else if (!tun && !is_ha_remote) {
         /* Remote port connected by localnet port */
-        /* Table 33, priority 100.
+        /* Table 38, priority 100.
          * =======================
          *
          * Implements switching to localnet port. Each flow matches a
@@ -1329,7 +1328,7 @@ consider_port_binding(struct ovsdb_idl_index *sbrec_port_binding_by_name,
 
         put_load(localnet_port->tunnel_key, MFF_LOG_OUTPORT, 0, 32, ofpacts_p);
 
-        /* Resubmit to table 33. */
+        /* Resubmit to table 38. */
         put_resubmit(OFTABLE_LOCAL_OUTPUT, ofpacts_p);
         ofctrl_add_flow(flow_table, OFTABLE_LOCAL_OUTPUT, 100,
                         binding->header_.uuid.parts[0],
@@ -1341,7 +1340,7 @@ consider_port_binding(struct ovsdb_idl_index *sbrec_port_binding_by_name,
 
         /* Remote port connected by tunnel */
 
-        /* Table 32, priority 100.
+        /* Table 38, priority 100.
          * =======================
          *
          * Handles traffic that needs to be sent to a remote hypervisor.  Each
@@ -1372,6 +1371,43 @@ out:
     }
 }
 
+static int64_t
+get_vxlan_port_key(int64_t port_key)
+{
+    if (port_key >= OVN_MIN_MULTICAST) {
+        /* 0b1<11 least significant bits> */
+        return OVN_VXLAN_MIN_MULTICAST |
+            (port_key & (OVN_VXLAN_MIN_MULTICAST - 1));
+    }
+    return port_key;
+}
+
+static void
+fanout_to_chassis(enum mf_field_id mff_ovn_geneve,
+                  struct sset *remote_chassis,
+                  const struct hmap *chassis_tunnels,
+                  const struct sbrec_datapath_binding *datapath,
+                  uint16_t outport, bool is_ramp_switch,
+                  struct ofpbuf *remote_ofpacts)
+{
+    const char *chassis_name;
+    const struct chassis_tunnel *prev = NULL;
+    SSET_FOR_EACH (chassis_name, remote_chassis) {
+        const struct chassis_tunnel *tun
+            = chassis_tunnel_find(chassis_tunnels, chassis_name, NULL);
+        if (!tun) {
+            continue;
+        }
+
+        if (!prev || tun->type != prev->type) {
+            put_encapsulation(mff_ovn_geneve, tun, datapath,
+                              outport, is_ramp_switch, remote_ofpacts);
+            prev = tun;
+        }
+        ofpact_put_OUTPUT(remote_ofpacts)->port = tun->ofport;
+    }
+}
+
 static void
 consider_mc_group(struct ovsdb_idl_index *sbrec_port_binding_by_name,
                   enum mf_field_id mff_ovn_geneve,
@@ -1385,11 +1421,13 @@ consider_mc_group(struct ovsdb_idl_index *sbrec_port_binding_by_name,
                   struct ovn_desired_flow_table *flow_table)
 {
     uint32_t dp_key = mc->datapath->tunnel_key;
-    if (!get_local_datapath(local_datapaths, dp_key)) {
+    struct local_datapath *ldp = get_local_datapath(local_datapaths, dp_key);
+    if (!ldp) {
         return;
     }
 
     struct sset remote_chassis = SSET_INITIALIZER(&remote_chassis);
+    struct sset vtep_chassis = SSET_INITIALIZER(&vtep_chassis);
     struct match match;
 
     match_init_catchall(&match);
@@ -1398,7 +1436,8 @@ consider_mc_group(struct ovsdb_idl_index *sbrec_port_binding_by_name,
 
     /* Go through all of the ports in the multicast group:
      *
-     *    - For remote ports, add the chassis to 'remote_chassis'.
+     *    - For remote ports, add the chassis to 'remote_chassis' or
+     *      'vtep_chassis'.
      *
      *    - For local ports (other than logical patch ports), add actions
      *      to 'ofpacts' to set the output port and resubmit.
@@ -1406,7 +1445,10 @@ consider_mc_group(struct ovsdb_idl_index *sbrec_port_binding_by_name,
      *    - For logical patch ports, add actions to 'remote_ofpacts'
      *      instead.  (If we put them in 'ofpacts', then the output
      *      would happen on every hypervisor in the multicast group,
-     *      effectively duplicating the packet.)
+     *      effectively duplicating the packet.)  The only exception
+     *      is in case of transit switches (used by OVN-IC).  We need
+     *      patch ports to be processed on the remote side, after
+     *      crossing the AZ boundary.
      *
      *    - For chassisredirect ports, add actions to 'ofpacts' to
      *      set the output port to be the router patch port for which
@@ -1436,13 +1478,25 @@ consider_mc_group(struct ovsdb_idl_index *sbrec_port_binding_by_name,
                                   port->parent_port : port->logical_port;
 
         if (!strcmp(port->type, "patch")) {
+            if (ldp->is_transit_switch) {
+                put_load(port->tunnel_key, MFF_LOG_OUTPORT, 0, 32,
+                         &ofpacts);
+                put_resubmit(OFTABLE_CHECK_LOOPBACK, &ofpacts);
+            } else {
+                put_load(port->tunnel_key, MFF_LOG_OUTPORT, 0, 32,
+                         &remote_ofpacts);
+                put_resubmit(OFTABLE_CHECK_LOOPBACK, &remote_ofpacts);
+            }
+        } else if (!strcmp(port->type, "localport")) {
             put_load(port->tunnel_key, MFF_LOG_OUTPORT, 0, 32,
                      &remote_ofpacts);
             put_resubmit(OFTABLE_CHECK_LOOPBACK, &remote_ofpacts);
-        } else if (local_binding_get_primary_pb(local_bindings, lport_name)
-                   || simap_contains(patch_ofports, port->logical_port)
-                   || (!strcmp(port->type, "l3gateway")
-                       && port->chassis == chassis)) {
+        } else if (port->chassis == chassis
+                   && (local_binding_get_primary_pb(local_bindings, lport_name)
+                       || !strcmp(port->type, "l3gateway"))) {
+            put_load(port->tunnel_key, MFF_LOG_OUTPORT, 0, 32, &ofpacts);
+            put_resubmit(OFTABLE_CHECK_LOOPBACK, &ofpacts);
+        } else if (simap_contains(patch_ofports, port->logical_port)) {
             put_load(port->tunnel_key, MFF_LOG_OUTPORT, 0, 32, &ofpacts);
             put_resubmit(OFTABLE_CHECK_LOOPBACK, &ofpacts);
         } else if (!strcmp(port->type, "chassisredirect")
@@ -1465,11 +1519,16 @@ consider_mc_group(struct ovsdb_idl_index *sbrec_port_binding_by_name,
             /* Add remote chassis only when localnet port not exist,
              * otherwise multicast will reach remote ports through localnet
              * port. */
-            sset_add(&remote_chassis, port->chassis->name);
+            if (smap_get_bool(&port->chassis->other_config,
+                              "is-vtep", false)) {
+                sset_add(&vtep_chassis, port->chassis->name);
+            } else {
+                sset_add(&remote_chassis, port->chassis->name);
+            }
         }
     }
 
-    /* Table 33, priority 100.
+    /* Table 38, priority 100.
      * =======================
      *
      * Handle output to the local logical ports in the multicast group, if
@@ -1485,12 +1544,13 @@ consider_mc_group(struct ovsdb_idl_index *sbrec_port_binding_by_name,
                         &match, &ofpacts, &mc->header_.uuid);
     }
 
-    /* Table 32, priority 100.
+    /* Table 37, priority 100.
      * =======================
      *
      * Handle output to the remote chassis in the multicast group, if
      * any. */
-    if (!sset_is_empty(&remote_chassis) || remote_ofpacts.size > 0) {
+    if (!sset_is_empty(&remote_chassis) ||
+            !sset_is_empty(&vtep_chassis) || remote_ofpacts.size > 0) {
         if (remote_ofpacts.size > 0) {
             /* Following delivery to logical patch ports, restore the
              * multicast group as the logical output port. */
@@ -1498,22 +1558,12 @@ consider_mc_group(struct ovsdb_idl_index *sbrec_port_binding_by_name,
                      &remote_ofpacts);
         }
 
-        const char *chassis_name;
-        const struct chassis_tunnel *prev = NULL;
-        SSET_FOR_EACH (chassis_name, &remote_chassis) {
-            const struct chassis_tunnel *tun
-                = chassis_tunnel_find(chassis_tunnels, chassis_name, NULL);
-            if (!tun) {
-                continue;
-            }
-
-            if (!prev || tun->type != prev->type) {
-                put_encapsulation(mff_ovn_geneve, tun, mc->datapath,
-                                  mc->tunnel_key, true, &remote_ofpacts);
-                prev = tun;
-            }
-            ofpact_put_OUTPUT(&remote_ofpacts)->port = tun->ofport;
-        }
+        fanout_to_chassis(mff_ovn_geneve, &remote_chassis, chassis_tunnels,
+                          mc->datapath, mc->tunnel_key, false,
+                          &remote_ofpacts);
+        fanout_to_chassis(mff_ovn_geneve, &vtep_chassis, chassis_tunnels,
+                          mc->datapath, mc->tunnel_key, true,
+                          &remote_ofpacts);
 
         if (remote_ofpacts.size) {
             if (local_ports) {
@@ -1527,6 +1577,25 @@ consider_mc_group(struct ovsdb_idl_index *sbrec_port_binding_by_name,
     ofpbuf_uninit(&ofpacts);
     ofpbuf_uninit(&remote_ofpacts);
     sset_destroy(&remote_chassis);
+    sset_destroy(&vtep_chassis);
+}
+
+static void
+physical_eval_port_binding(struct physical_ctx *p_ctx,
+                           const struct sbrec_port_binding *pb,
+                           struct ovn_desired_flow_table *flow_table)
+{
+    struct ofpbuf ofpacts;
+    ofpbuf_init(&ofpacts, 0);
+    consider_port_binding(p_ctx->sbrec_port_binding_by_name,
+                          p_ctx->mff_ovn_geneve, p_ctx->ct_zones,
+                          p_ctx->active_tunnels,
+                          p_ctx->local_datapaths,
+                          p_ctx->local_bindings,
+                          p_ctx->patch_ofports,
+                          p_ctx->chassis_tunnels,
+                          pb, p_ctx->chassis, flow_table, &ofpacts);
+    ofpbuf_uninit(&ofpacts);
 }
 
 bool
@@ -1550,33 +1619,20 @@ physical_handle_flows_for_lport(const struct sbrec_port_binding *pb,
             get_local_datapath(p_ctx->local_datapaths,
                                pb->datapath->tunnel_key);
         if (ldp && ldp->localnet_port) {
-            struct ofpbuf ofpacts;
             ofctrl_remove_flows(flow_table, &ldp->localnet_port->header_.uuid);
-            ofpbuf_init(&ofpacts, 0);
-            consider_port_binding(p_ctx->sbrec_port_binding_by_name,
-                                  p_ctx->mff_ovn_geneve, p_ctx->ct_zones,
-                                  p_ctx->active_tunnels,
-                                  p_ctx->local_datapaths,
-                                  p_ctx->local_bindings,
-                                  p_ctx->patch_ofports,
-                                  p_ctx->chassis_tunnels,
-                                  ldp->localnet_port, p_ctx->chassis,
-                                  flow_table, &ofpacts);
-            ofpbuf_uninit(&ofpacts);
+            physical_eval_port_binding(p_ctx, ldp->localnet_port, flow_table);
         }
     }
 
     if (!removed) {
-        struct ofpbuf ofpacts;
-        ofpbuf_init(&ofpacts, 0);
-        consider_port_binding(p_ctx->sbrec_port_binding_by_name,
-                              p_ctx->mff_ovn_geneve, p_ctx->ct_zones,
-                              p_ctx->active_tunnels, p_ctx->local_datapaths,
-                              p_ctx->local_bindings,
-                              p_ctx->patch_ofports,
-                              p_ctx->chassis_tunnels, pb,
-                              p_ctx->chassis, flow_table, &ofpacts);
-        ofpbuf_uninit(&ofpacts);
+        physical_eval_port_binding(p_ctx, pb, flow_table);
+        if (!strcmp(pb->type, "patch")) {
+            const struct sbrec_port_binding *peer =
+                get_binding_peer(p_ctx->sbrec_port_binding_by_name, pb);
+            if (peer) {
+                physical_eval_port_binding(p_ctx, peer, flow_table);
+            }
+        }
     }
 
     return true;
@@ -1635,7 +1691,7 @@ physical_run(struct physical_ctx *p_ctx,
                               p_ctx->chassis, flow_table, &ofpacts);
     }
 
-    /* Handle output to multicast groups, in tables 32 and 33. */
+    /* Handle output to multicast groups, in tables 37 and 38. */
     const struct sbrec_multicast_group *mc;
     SBREC_MULTICAST_GROUP_TABLE_FOR_EACH (mc, p_ctx->mc_group_table) {
         consider_mc_group(p_ctx->sbrec_port_binding_by_name,
@@ -1656,7 +1712,7 @@ physical_run(struct physical_ctx *p_ctx,
      * encapsulations have metadata about the ingress and egress logical ports.
      * VXLAN encapsulations have metadata about the egress logical port only.
      * We set MFF_LOG_DATAPATH, MFF_LOG_INPORT, and MFF_LOG_OUTPORT from the
-     * tunnel key data where possible, then resubmit to table 33 to handle
+     * tunnel key data where possible, then resubmit to table 38 to handle
      * packets to the local hypervisor. */
     struct chassis_tunnel *tun;
     HMAP_FOR_EACH (tun, hmap_node, p_ctx->chassis_tunnels) {
@@ -1689,6 +1745,27 @@ physical_run(struct physical_ctx *p_ctx,
                         &ofpacts, hc_uuid);
     }
 
+    /* Add VXLAN specific rules to transform port keys
+     * from 12 bits to 16 bits used elsewhere. */
+    HMAP_FOR_EACH (tun, hmap_node, p_ctx->chassis_tunnels) {
+        if (tun->type == VXLAN) {
+            ofpbuf_clear(&ofpacts);
+
+            struct match match = MATCH_CATCHALL_INITIALIZER;
+            match_set_in_port(&match, tun->ofport);
+            ovs_be64 mcast_bits = htonll((OVN_VXLAN_MIN_MULTICAST << 12));
+            match_set_tun_id_masked(&match, mcast_bits, mcast_bits);
+
+            put_load(1, MFF_LOG_OUTPORT, 15, 1, &ofpacts);
+            put_move(MFF_TUN_ID, 12, MFF_LOG_OUTPORT,  0, 11, &ofpacts);
+            put_move(MFF_TUN_ID, 0, MFF_LOG_DATAPATH, 0, 12, &ofpacts);
+            put_resubmit(OFTABLE_LOCAL_OUTPUT, &ofpacts);
+
+            ofctrl_add_flow(flow_table, OFTABLE_PHY_TO_LOG, 105, 0,
+                            &match, &ofpacts, hc_uuid);
+        }
+    }
+
     /* Handle ramp switch encapsulations. */
     HMAP_FOR_EACH (tun, hmap_node, p_ctx->chassis_tunnels) {
         if (tun->type != VXLAN) {
@@ -1730,12 +1807,12 @@ physical_run(struct physical_ctx *p_ctx,
         }
     }
 
-    /* Table 32, priority 150.
+    /* Table 37, priority 150.
      * =======================
      *
      * Handles packets received from a VXLAN tunnel which get resubmitted to
      * OFTABLE_LOG_INGRESS_PIPELINE due to lack of needed metadata in VXLAN,
-     * explicitly skip sending back out any tunnels and resubmit to table 33
+     * explicitly skip sending back out any tunnels and resubmit to table 38
      * for local delivery.
      */
     struct match match;
@@ -1743,13 +1820,13 @@ physical_run(struct physical_ctx *p_ctx,
     match_set_reg_masked(&match, MFF_LOG_FLAGS - MFF_REG0,
                          MLF_RCV_FROM_RAMP, MLF_RCV_FROM_RAMP);
 
-    /* Resubmit to table 33. */
+    /* Resubmit to table 38. */
     ofpbuf_clear(&ofpacts);
     put_resubmit(OFTABLE_LOCAL_OUTPUT, &ofpacts);
     ofctrl_add_flow(flow_table, OFTABLE_REMOTE_OUTPUT, 150, 0,
                     &match, &ofpacts, hc_uuid);
 
-    /* Table 32, priority 150.
+    /* Table 37, priority 150.
      * =======================
      *
      * Packets that should not be sent to other hypervisors.
@@ -1757,19 +1834,19 @@ physical_run(struct physical_ctx *p_ctx,
     match_init_catchall(&match);
     match_set_reg_masked(&match, MFF_LOG_FLAGS - MFF_REG0,
                          MLF_LOCAL_ONLY, MLF_LOCAL_ONLY);
-    /* Resubmit to table 33. */
+    /* Resubmit to table 38. */
     ofpbuf_clear(&ofpacts);
     put_resubmit(OFTABLE_LOCAL_OUTPUT, &ofpacts);
     ofctrl_add_flow(flow_table, OFTABLE_REMOTE_OUTPUT, 150, 0,
                     &match, &ofpacts, hc_uuid);
 
-    /* Table 32, priority 150.
+    /* Table 37, priority 150.
      * =======================
      *
      * Handles packets received from ports of type "localport".  These ports
      * are present on every hypervisor.  Traffic that originates at one should
      * never go over a tunnel to a remote hypervisor, so resubmit them to table
-     * 33 for local delivery. */
+     * 38 for local delivery. */
     match_init_catchall(&match);
     ofpbuf_clear(&ofpacts);
     put_resubmit(OFTABLE_LOCAL_OUTPUT, &ofpacts);
@@ -1789,7 +1866,7 @@ physical_run(struct physical_ctx *p_ctx,
         }
     }
 
-    /* Table 32, Priority 0.
+    /* Table 37, Priority 0.
      * =======================
      *
      * Resubmit packets that are not directed at tunnels or part of a
@@ -1800,11 +1877,11 @@ physical_run(struct physical_ctx *p_ctx,
     ofctrl_add_flow(flow_table, OFTABLE_REMOTE_OUTPUT, 0, 0, &match,
                     &ofpacts, hc_uuid);
 
-    /* Table 34, Priority 0.
+    /* Table 39, Priority 0.
      * =======================
      *
      * Resubmit packets that don't output to the ingress port (already checked
-     * in table 33) to the logical egress pipeline, clearing the logical
+     * in table 38) to the logical egress pipeline, clearing the logical
      * registers (for consistent behavior with packets that get tunneled). */
     match_init_catchall(&match);
     ofpbuf_clear(&ofpacts);
