@@ -66,6 +66,8 @@ struct ovn_extend_table;
     OVNACT(CT_COMMIT_V2,      ovnact_nest)            \
     OVNACT(CT_DNAT,           ovnact_ct_nat)          \
     OVNACT(CT_SNAT,           ovnact_ct_nat)          \
+    OVNACT(CT_DNAT_IN_CZONE,  ovnact_ct_nat)          \
+    OVNACT(CT_SNAT_IN_CZONE,  ovnact_ct_nat)          \
     OVNACT(CT_LB,             ovnact_ct_lb)           \
     OVNACT(SELECT,            ovnact_select)          \
     OVNACT(CT_CLEAR,          ovnact_null)            \
@@ -804,5 +806,7 @@ void ovnacts_encode(const struct ovnact[], size_t ovnacts_len,
                     struct ofpbuf *ofpacts);
 
 void ovnacts_free(struct ovnact[], size_t ovnacts_len);
+char *ovnact_op_to_string(uint32_t);
+int encode_ra_dnssl_opt(char *data, char *buf, int buf_len);
 
 #endif /* ovn/actions.h */
