@@ -25,8 +25,11 @@
 #include "lib/util.h"
 #include "openvswitch/dynamic-string.h"
 #include "openvswitch/hmap.h"
+#include "openvswitch/poll-loop.h"
 #include "openvswitch/vlog.h"
 #include "inc-proc-eng.h"
+#include "timeval.h"
+#include "unixctl.h"
 
 VLOG_DEFINE_THIS_MODULE(inc_proc_eng);
 
@@ -38,12 +41,18 @@ static struct engine_node **engine_nodes;
 static size_t engine_n_nodes;
 
 static const char *engine_node_state_name[EN_STATE_MAX] = {
-    [EN_STALE]   = "Stale",
-    [EN_UPDATED] = "Updated",
-    [EN_VALID]   = "Valid",
-    [EN_ABORTED] = "Aborted",
+    [EN_STALE]     = "Stale",
+    [EN_UPDATED]   = "Updated",
+    [EN_UNCHANGED] = "Unchanged",
+    [EN_ABORTED]   = "Aborted",
 };
 
+static long long engine_compute_log_timeout_msec = 500;
+
+static void
+engine_recompute(struct engine_node *node, bool allowed,
+                 const char *reason_fmt, ...) OVS_PRINTF_FORMAT(3, 4);
+
 void
 engine_set_force_recompute(bool val)
 {
@@ -102,6 +111,63 @@ engine_get_nodes(struct engine_node *node, size_t *n_count)
     return engine_topo_sort(node, NULL, n_count, &n_size);
 }
 
+static void
+engine_clear_stats(struct unixctl_conn *conn, int argc OVS_UNUSED,
+                   const char *argv[] OVS_UNUSED, void *arg OVS_UNUSED)
+{
+    for (size_t i = 0; i < engine_n_nodes; i++) {
+        struct engine_node *node = engine_nodes[i];
+
+        memset(&node->stats, 0, sizeof node->stats);
+    }
+    unixctl_command_reply(conn, NULL);
+}
+
+static void
+engine_dump_stats(struct unixctl_conn *conn, int argc OVS_UNUSED,
+                  const char *argv[] OVS_UNUSED, void *arg OVS_UNUSED)
+{
+    struct ds dump = DS_EMPTY_INITIALIZER;
+
+    for (size_t i = 0; i < engine_n_nodes; i++) {
+        struct engine_node *node = engine_nodes[i];
+
+        ds_put_format(&dump,
+                      "Node: %s\n"
+                      "- recompute: %12"PRIu64"\n"
+                      "- compute:   %12"PRIu64"\n"
+                      "- abort:     %12"PRIu64"\n",
+                      node->name, node->stats.recompute,
+                      node->stats.compute, node->stats.abort);
+    }
+    unixctl_command_reply(conn, ds_cstr(&dump));
+
+    ds_destroy(&dump);
+}
+
+static void
+engine_trigger_recompute_cmd(struct unixctl_conn *conn, int argc OVS_UNUSED,
+                             const char *argv[] OVS_UNUSED,
+                             void *arg OVS_UNUSED)
+{
+    engine_trigger_recompute();
+    unixctl_command_reply(conn, NULL);
+}
+
+static void
+engine_set_log_timeout_cmd(struct unixctl_conn *conn, int argc OVS_UNUSED,
+                           const char *argv[], void *arg OVS_UNUSED)
+{
+
+    unsigned int timeout;
+    if (!str_to_uint(argv[1], 10, &timeout)) {
+        unixctl_command_reply_error(conn, "unsigned integer required");
+        return;
+    }
+    engine_compute_log_timeout_msec = timeout;
+    unixctl_command_reply(conn, NULL);
+}
+
 void
 engine_init(struct engine_node *node, struct engine_arg *arg)
 {
@@ -115,6 +181,15 @@ engine_init(struct engine_node *node, struct engine_arg *arg)
             engine_nodes[i]->data = NULL;
         }
     }
+
+    unixctl_command_register("inc-engine/show-stats", "", 0, 0,
+                             engine_dump_stats, NULL);
+    unixctl_command_register("inc-engine/clear-stats", "", 0, 0,
+                             engine_clear_stats, NULL);
+    unixctl_command_register("inc-engine/recompute", "", 0, 0,
+                             engine_trigger_recompute_cmd, NULL);
+    unixctl_command_register("inc-engine/compute-log-timeout", "", 1, 1,
+                             engine_set_log_timeout_cmd, NULL);
 }
 
 void
@@ -210,7 +285,7 @@ engine_set_node_state_at(struct engine_node *node,
 static bool
 engine_node_valid(struct engine_node *node)
 {
-    if (node->state == EN_UPDATED || node->state == EN_VALID) {
+    if (node->state == EN_UPDATED || node->state == EN_UNCHANGED) {
         return true;
     }
 
@@ -275,19 +350,46 @@ engine_init_run(void)
  * mark the node as "aborted".
  */
 static void
-engine_recompute(struct engine_node *node, bool forced, bool allowed)
+engine_recompute(struct engine_node *node, bool allowed,
+                 const char *reason_fmt, ...)
 {
-    VLOG_DBG("node: %s, recompute (%s)", node->name,
-             forced ? "forced" : "triggered");
+    char *reason = NULL;
+
+    if (VLOG_IS_DBG_ENABLED()) {
+        va_list reason_args;
+
+        va_start(reason_args, reason_fmt);
+        reason = xvasprintf(reason_fmt, reason_args);
+        va_end(reason_args);
+    }
 
     if (!allowed) {
-        VLOG_DBG("node: %s, recompute aborted", node->name);
+        VLOG_DBG("node: %s, recompute (%s) aborted", node->name, reason);
         engine_set_node_state(node, EN_ABORTED);
-        return;
+        goto done;
+    }
+
+    /* Clear tracked data before calling run() so that partially tracked data
+     * from some of the change handler executions are cleared. */
+    if (node->clear_tracked_data) {
+        node->clear_tracked_data(node->data);
     }
 
     /* Run the node handler which might change state. */
+    long long int now = time_msec();
     node->run(node, node->data);
+    node->stats.recompute++;
+    long long int delta_time = time_msec() - now;
+    if (delta_time > engine_compute_log_timeout_msec) {
+        static struct vlog_rate_limit rl = VLOG_RATE_LIMIT_INIT(20, 10);
+        VLOG_INFO_RL(&rl, "node: %s, recompute (%s) took %lldms", node->name,
+                     reason, delta_time);
+    } else {
+        VLOG_DBG("node: %s, recompute (%s) took %lldms", node->name, reason,
+                 delta_time);
+    }
+done:
+    free(reason);
 }
 
 /* Return true if the node could be computed, false otherwise. */
@@ -297,21 +399,32 @@ engine_compute(struct engine_node *node, bool recompute_allowed)
     for (size_t i = 0; i < node->n_inputs; i++) {
         /* If the input node data changed call its change handler. */
         if (node->inputs[i].node->state == EN_UPDATED) {
-            VLOG_DBG("node: %s, handle change for input %s",
-                     node->name, node->inputs[i].node->name);
-
             /* If the input change can't be handled incrementally, run
              * the node handler.
              */
-            if (!node->inputs[i].change_handler(node, node->data)) {
-                VLOG_DBG("node: %s, can't handle change for input %s, "
-                         "fall back to recompute",
-                         node->name, node->inputs[i].node->name);
-                engine_recompute(node, false, recompute_allowed);
+            long long int now = time_msec();
+            bool handled = node->inputs[i].change_handler(node, node->data);
+            long long int delta_time = time_msec() - now;
+            if (delta_time > engine_compute_log_timeout_msec) {
+                static struct vlog_rate_limit rl =
+                    VLOG_RATE_LIMIT_INIT(20, 10);
+                VLOG_INFO_RL(&rl, "node: %s, handler for input %s took %lldms",
+                             node->name, node->inputs[i].node->name,
+                             delta_time);
+            } else {
+                VLOG_DBG("node: %s, handler for input %s took %lldms",
+                         node->name, node->inputs[i].node->name, delta_time);
+            }
+            if (!handled) {
+                engine_recompute(node, recompute_allowed,
+                                 "failed handler for input %s",
+                                 node->inputs[i].node->name);
                 return (node->state != EN_ABORTED);
             }
         }
     }
+    node->stats.compute++;
+
     return true;
 }
 
@@ -321,11 +434,12 @@ engine_run_node(struct engine_node *node, bool recompute_allowed)
     if (!node->n_inputs) {
         /* Run the node handler which might change state. */
         node->run(node, node->data);
+        node->stats.recompute++;
         return;
     }
 
     if (engine_force_recompute) {
-        engine_recompute(node, true, recompute_allowed);
+        engine_recompute(node, recompute_allowed, "forced");
         return;
     }
 
@@ -339,7 +453,9 @@ engine_run_node(struct engine_node *node, bool recompute_allowed)
 
             /* Trigger a recompute if we don't have a change handler. */
             if (!node->inputs[i].change_handler) {
-                engine_recompute(node, false, recompute_allowed);
+                engine_recompute(node, recompute_allowed,
+                                 "missing handler for input %s",
+                                 node->inputs[i].node->name);
                 return;
             }
         }
@@ -358,7 +474,7 @@ engine_run_node(struct engine_node *node, bool recompute_allowed)
      * still valid.
      */
     if (!engine_node_changed(node)) {
-        engine_set_node_state(node, EN_VALID);
+        engine_set_node_state(node, EN_UNCHANGED);
     }
 }
 
@@ -377,6 +493,7 @@ engine_run(bool recompute_allowed)
         engine_run_node(engine_nodes[i], recompute_allowed);
 
         if (engine_nodes[i]->state == EN_ABORTED) {
+            engine_nodes[i]->stats.abort++;
             engine_run_aborted = true;
             return;
         }
@@ -393,6 +510,7 @@ engine_need_run(void)
         }
 
         engine_nodes[i]->run(engine_nodes[i], engine_nodes[i]->data);
+        engine_nodes[i]->stats.recompute++;
         VLOG_DBG("input node: %s, state: %s", engine_nodes[i]->name,
                  engine_node_state_name[engine_nodes[i]->state]);
         if (engine_nodes[i]->state == EN_UPDATED) {
@@ -401,3 +519,11 @@ engine_need_run(void)
     }
     return false;
 }
+
+void
+engine_trigger_recompute(void)
+{
+    VLOG_INFO("User triggered force recompute.");
+    engine_set_force_recompute(true);
+    poll_immediate_wake();
+}
