@@ -63,15 +63,23 @@
 #define ENGINE_MAX_INPUT 256
 #define ENGINE_MAX_OVSDB_INDEX 256
 
+#include <stdbool.h>
+#include <stdint.h>
+
+#include "compiler.h"
+
 struct engine_context {
     struct ovsdb_idl_txn *ovs_idl_txn;
     struct ovsdb_idl_txn *ovnsb_idl_txn;
+    struct ovsdb_idl_txn *ovnnb_idl_txn;
+
     void *client_ctx;
 };
 
 /* Arguments to be passed to the engine at engine_init(). */
 struct engine_arg {
     struct ovsdb_idl *sb_idl;
+    struct ovsdb_idl *nb_idl;
     struct ovsdb_idl *ovs_idl;
 };
 
@@ -98,7 +106,7 @@ enum engine_node_state {
     EN_UPDATED,   /* Data in the node is valid but was updated during the
                    * last run.
                    */
-    EN_VALID,     /* Data in the node is valid and didn't change during the
+    EN_UNCHANGED, /* Data in the node is valid and didn't change during the
                    * last run.
                    */
     EN_ABORTED,   /* During the last run, processing was aborted for
@@ -107,6 +115,12 @@ enum engine_node_state {
     EN_STATE_MAX,
 };
 
+struct engine_stats {
+    uint64_t recompute;
+    uint64_t compute;
+    uint64_t abort;
+};
+
 struct engine_node {
     /* A unique name for each node. */
     char *name;
@@ -154,6 +168,9 @@ struct engine_node {
     /* Method to clear up tracked data maintained by the engine node in the
      * engine 'data'. It may be NULL. */
     void (*clear_tracked_data)(void *tracked_data);
+
+    /* Engine stats. */
+    struct engine_stats stats;
 };
 
 /* Initialize the data for the engine nodes. It calls each node's
@@ -247,6 +264,9 @@ void *engine_get_internal_data(struct engine_node *node);
 #define engine_set_node_state(node, state) \
     engine_set_node_state_at(node, state, OVS_SOURCE_LOCATOR)
 
+/* Trigger a full recompute. */
+void engine_trigger_recompute(void);
+
 struct ed_ovsdb_index {
     const char *name;
     struct ovsdb_idl_index *index;
@@ -286,15 +306,16 @@ void engine_ovsdb_node_add_index(struct engine_node *, const char *name,
         .init = en_##NAME##_init, \
         .run = en_##NAME##_run, \
         .cleanup = en_##NAME##_cleanup, \
-        .is_valid = en_##NAME##_is_valid, \
+        .is_valid = NULL, \
         .clear_tracked_data = NULL, \
     };
 
-#define ENGINE_NODE_CUSTOM_DATA(NAME, NAME_STR) \
-    ENGINE_NODE_DEF(NAME, NAME_STR)
+#define ENGINE_NODE_WITH_CLEAR_TRACK_DATA_IS_VALID(NAME, NAME_STR) \
+    ENGINE_NODE(NAME, NAME_STR) \
+    en_##NAME.clear_tracked_data = en_##NAME##_clear_tracked_data; \
+    en_##NAME.is_valid = en_##NAME##_is_valid;
 
 #define ENGINE_NODE(NAME, NAME_STR) \
-    static bool (*en_##NAME##_is_valid)(struct engine_node *node) = NULL; \
     ENGINE_NODE_DEF(NAME, NAME_STR)
 
 #define ENGINE_NODE_WITH_CLEAR_TRACK_DATA(NAME, NAME_STR) \
@@ -314,7 +335,7 @@ en_##DB_NAME##_##TBL_NAME##_run(struct engine_node *node, \
         engine_set_node_state(node, EN_UPDATED); \
         return; \
     } \
-    engine_set_node_state(node, EN_VALID); \
+    engine_set_node_state(node, EN_UNCHANGED); \
 } \
 static void *en_##DB_NAME##_##TBL_NAME##_init( \
     struct engine_node *node OVS_UNUSED, \
@@ -334,6 +355,11 @@ static void en_##DB_NAME##_##TBL_NAME##_cleanup(void *data OVS_UNUSED) \
 #define ENGINE_FUNC_SB(TBL_NAME) \
     ENGINE_FUNC_OVSDB(sb, TBL_NAME)
 
+/* Macro to define member functions of an engine node which represents
+ * a table of OVN NB DB */
+#define ENGINE_FUNC_NB(TBL_NAME) \
+    ENGINE_FUNC_OVSDB(nb, TBL_NAME)
+
 /* Macro to define member functions of an engine node which represents
  * a table of open_vswitch DB */
 #define ENGINE_FUNC_OVS(TBL_NAME) \
@@ -347,6 +373,10 @@ static void en_##DB_NAME##_##TBL_NAME##_cleanup(void *data OVS_UNUSED) \
 #define ENGINE_NODE_SB(TBL_NAME, TBL_NAME_STR) \
     ENGINE_NODE_OVSDB(sb, "SB", TBL_NAME, TBL_NAME_STR);
 
+/* Macro to define an engine node which represents a table of OVN NB DB */
+#define ENGINE_NODE_NB(TBL_NAME, TBL_NAME_STR) \
+    ENGINE_NODE_OVSDB(nb, "NB", TBL_NAME, TBL_NAME_STR);
+
 /* Macro to define an engine node which represents a table of open_vswitch
  * DB */
 #define ENGINE_NODE_OVS(TBL_NAME, TBL_NAME_STR) \
