@@ -23,6 +23,7 @@
 #include "include/ovn/actions.h"
 #include "openvswitch/ofp-parse.h"
 #include "openvswitch/vlog.h"
+#include "lib/vswitch-idl.h"
 #include "ovn-dirs.h"
 #include "ovn-nb-idl.h"
 #include "ovn-sb-idl.h"
@@ -232,6 +233,27 @@ extract_ip_addresses(const char *address, struct lport_addresses *laddrs)
     return false;
 }
 
+/* Extracts at most one IPv4 and at most one IPv6 address from 'address'
+ * which should be of the format 'IP1 [IP2]'.
+ *
+ * Return true if at most one IPv4 address and at most one IPv6 address
+ * is found in 'address'.  IPs must be host IPs, i.e., no unmasked bits.
+ *
+ * The caller must call destroy_lport_addresses().
+ */
+bool extract_ip_address(const char *address, struct lport_addresses *laddrs)
+{
+    if (!extract_ip_addresses(address, laddrs) ||
+            laddrs->n_ipv4_addrs > 1 ||
+            laddrs->n_ipv6_addrs > 1 ||
+            (laddrs->n_ipv4_addrs && laddrs->ipv4_addrs[0].plen != 32) ||
+            (laddrs->n_ipv6_addrs && laddrs->ipv6_addrs[0].plen != 128)) {
+        destroy_lport_addresses(laddrs);
+        return false;
+    }
+    return true;
+}
+
 /* Extracts the mac, IPv4 and IPv6 addresses from the
  * "nbrec_logical_router_port" parameter 'lrp'.  Stores the IPv4 and
  * IPv6 addresses in the 'ipv4_addrs' and 'ipv6_addrs' fields of
@@ -502,11 +524,18 @@ ovn_is_known_nb_lsp_type(const char *type)
     return false;
 }
 
+static enum ovn_pipeline
+ovn_pipeline_from_name(const char *pipeline)
+{
+    return pipeline[0] == 'i' ? P_IN : P_OUT;
+}
+
 uint32_t
 sbrec_logical_flow_hash(const struct sbrec_logical_flow *lf)
 {
     const struct sbrec_datapath_binding *ld = lf->logical_datapath;
-    uint32_t hash = ovn_logical_flow_hash(lf->table_id, lf->pipeline,
+    uint32_t hash = ovn_logical_flow_hash(lf->table_id,
+                                          ovn_pipeline_from_name(lf->pipeline),
                                           lf->priority, lf->match,
                                           lf->actions);
 
@@ -514,12 +543,11 @@ sbrec_logical_flow_hash(const struct sbrec_logical_flow *lf)
 }
 
 uint32_t
-ovn_logical_flow_hash(uint8_t table_id, const char *pipeline,
+ovn_logical_flow_hash(uint8_t table_id, enum ovn_pipeline pipeline,
                       uint16_t priority,
                       const char *match, const char *actions)
 {
-    size_t hash = hash_2words((table_id << 16) | priority, 0);
-    hash = hash_string(pipeline, hash);
+    size_t hash = hash_2words((table_id << 16) | priority, pipeline);
     hash = hash_string(match, hash);
     return hash_string(actions, hash);
 }
@@ -531,18 +559,6 @@ ovn_logical_flow_hash_datapath(const struct uuid *logical_datapath,
     return hash_add(hash, uuid_hash(logical_datapath));
 }
 
-bool
-datapath_is_switch(const struct sbrec_datapath_binding *ldp)
-{
-    return smap_get(&ldp->external_ids, "logical-switch") != NULL;
-}
-
-int
-datapath_snat_ct_zone(const struct sbrec_datapath_binding *dp)
-{
-    return smap_get_int(&dp->external_ids, "snat-ct-zone", -1);
-}
-
 
 struct tnlid_node {
     struct hmap_node hmap_node;
@@ -559,18 +575,30 @@ ovn_destroy_tnlids(struct hmap *tnlids)
     hmap_destroy(tnlids);
 }
 
+/* Returns true if 'tnlid' is present in the hmap 'tnlids'. */
 bool
-ovn_add_tnlid(struct hmap *set, uint32_t tnlid)
+ovn_tnlid_present(struct hmap *tnlids, uint32_t tnlid)
 {
     uint32_t hash = hash_int(tnlid, 0);
     struct tnlid_node *node;
-    HMAP_FOR_EACH_IN_BUCKET (node, hmap_node, hash, set) {
+    HMAP_FOR_EACH_IN_BUCKET (node, hmap_node, hash, tnlids) {
         if (node->tnlid == tnlid) {
-            return false;
+            return true;
         }
     }
 
-    node = xmalloc(sizeof *node);
+    return false;
+}
+
+bool
+ovn_add_tnlid(struct hmap *set, uint32_t tnlid)
+{
+    if (ovn_tnlid_present(set, tnlid)) {
+        return false;
+    }
+
+    uint32_t hash = hash_int(tnlid, 0);
+    struct tnlid_node *node = xmalloc(sizeof *node);
     hmap_insert(set, &node->hmap_node, hash);
     node->tnlid = tnlid;
     return true;
@@ -725,9 +753,9 @@ ip_address_and_port_from_lb_key(const char *key, char **ip_address,
     return true;
 }
 
-/* Increment this for any logical flow changes or if existing OVN action is
- * modified. */
-#define OVN_INTERNAL_MINOR_VER 0
+/* Increment this for any logical flow changes, if an existing OVN action is
+ * modified or a stage is added to a logical pipeline. */
+#define OVN_INTERNAL_MINOR_VER 3
 
 /* Returns the OVN version. The caller must free the returned value. */
 char *
@@ -737,3 +765,44 @@ ovn_get_internal_version(void)
                      sbrec_get_db_version(),
                      N_OVNACTS, OVN_INTERNAL_MINOR_VER);
 }
+
+#ifdef DDLOG
+/* Callbacks used by the ddlog northd code to print warnings and errors. */
+void
+ddlog_warn(const char *msg)
+{
+    VLOG_WARN("%s", msg);
+}
+
+void
+ddlog_err(const char *msg)
+{
+    VLOG_ERR("%s", msg);
+}
+#endif
+
+uint32_t
+get_tunnel_type(const char *name)
+{
+    if (!strcmp(name, "geneve")) {
+        return GENEVE;
+    } else if (!strcmp(name, "stt")) {
+        return STT;
+    } else if (!strcmp(name, "vxlan")) {
+        return VXLAN;
+    }
+
+    return 0;
+}
+
+const struct ovsrec_bridge *
+get_bridge(const struct ovsrec_bridge_table *bridge_table, const char *br_name)
+{
+    const struct ovsrec_bridge *br;
+    OVSREC_BRIDGE_TABLE_FOR_EACH (br, bridge_table) {
+        if (!strcmp(br->name, br_name)) {
+            return br;
+        }
+    }
+    return NULL;
+}
